#!/usr/bin/env python3

import asyncio

import aiocoap.resource as resource
import aiocoap
import re
import cbor
import link_header

CONTENT_FORMAT_CORE = 40
DEFAULT_PORT = 5683
global root

class RemoteResource(resource.Resource):
    def __init__(self, request):
        super(RemoteResource, self).__init__()
        print('RemoteResource init-ing with request ', request)
        print('Remote is ', request.remote.sockaddr[0])
        # We use this as the key in a map to this object
        self.remote_host = request.remote.sockaddr[0]
        self.host = request.opt.uri_host
        print('Host key is ', self.host)
        
        # Check if 'con' option is set
        self.url = self.ep = self.d = self.et = self.lt = None
        for query_elmt in  request.opt.uri_query:
            [tag, value] = query_elmt.split("=")
            print('  tag %s value %s)' % (tag, value))
            if tag == 'con':
                self.url = value
            elif tag == 'ep':
                self.ep = value
            elif tag == 'd':
                self.d = value
            elif tag == 'et':
                self.et = value
            elif tag == 'lt':
                self.lt = value

        # Fix up url if it wasn't set by 'con' option
        if self.url == None:
            port = request.opt.uri_port if request.opt.uri_port else DEFAULT_PORT
            url = 'coap:[' + self.remote_host + ']:' + str(port)

        # now setup the links, with non-local url
        if request.opt.content_format == 0:
            # utf-8
            payload = request.payload.decode('utf-8')
        elif request.opt.content_format == 60:
            # CBOR
            payload = cbor.loads(request.payload)
        else:
            # Duh repeats 0
            payload = request.payload.decode('utf-8')

        links = link_header.parse(payload)
        for link in links.links:
            # lose " from string - why do they come in?
            link.href = self.url.strip('"') + link.href
            print('Full url is now %s' % (link.href))
        
        # capture the CBOR representation of the resource
        self.payload = str(links)
        print('End of RemoteResource init')

class ResourceDirectory(resource.Resource):
    def __init__(self):
        super(ResourceDirectory, self).__init__()
        self.rt = 'core.rd'
        self.directory = {}
        
    def render_post(self, request):
        global root
        rr = RemoteResource(request)

        # add the resource with URL /rd/rr.host
        root.add_resource(('rd',rr.remote_host), rr)

        # add to our directory set
        self.directory[rr.host] = rr

        # return URL for updates to registering agent
        content = '/rd/' + rr.remote_host
        payload = content.encode('utf8')
        return aiocoap.Message(code=aiocoap.CHANGED, payload=payload)

        mesg.opt.content_format = CONTENT_FORMAT_CORE
        return mesg

class ResourceDirectoryLookup(resource.Resource):
    def __init__(self, resource_directory):
        super(ResourceDirectoryLookup, self).__init__()
        self.rt = 'core.rd-lookup'
        self.resource_directory = resource_directory

class ResourceDirectoryLookupResources(resource.Resource):
    def __init__(self, resource_directory):
        super(ResourceDirectoryLookupResources, self).__init__()
        self.resource_directory = resource_directory

    def render_get(self, request):
        payload = ''.encode('utf-8')
        for _, rr in self.resource_directory.directory.items():
            payload += (rr.payload + ',').encode('utf-8')
        # lose trailing ','
        payload = payload[:-1]
        
        # return the CBOR representation of the resource
        mesg = aiocoap.Message(code=aiocoap.CONTENT,
                               payload=payload)
        mesg.opt.content_format = CONTENT_FORMAT_CORE
        return mesg
        
def main():
    # Resource tree creation
    global root
    root = resource.Site()
    
    root.add_resource(('.well-known', 'core'),
                      resource.WKCResource(root.get_resources_as_linkheader))
    resource_directory = ResourceDirectory()
    root.add_resource(('rd',), resource_directory)
    
    # There should be separate lookup types for 'd', 'ep', 'res' and 'gp'
    # We only do 'rd-lookup' (with no methods) to appear in
    # the WK/C list and 'rd-lookup/res' as the one with a GET method
    root.add_resource(('rd-lookup',),
                      ResourceDirectoryLookup(resource_directory))
    root.add_resource(('rd-lookup','res'),
                      ResourceDirectoryLookupResources(resource_directory),
                      True)
    # There should be a group management handler
    # root.add_resource(('rd-group',), ResourceDirectoryGroup())
    
    asyncio.async(aiocoap.Context.create_server_context(root))

    asyncio.get_event_loop().run_forever()

if __name__ == "__main__":
    main()
