#!/usr/bin/env python3

import asyncio
from aiocoap import *
import link_header
import sys
import cbor

def get_resource_directory_urls(links):
    print("Printing links\n")
    rd_href = rd_lookup_href = rd_group_href = None
    for link in links.links:
        print("one link is ", link)
        print('href: ', link.href)
        for (attr, value) in link.attr_pairs:
            # print('attr ', attr, ' value ', value)
            if attr == 'rt':
                if value == 'core.rd':
                    rd_href = link.href
                elif value == 'core.rd-lookup':
                    rd_lookup_href = link.href
                elif  value == 'core.rd-group':
                    rd_group_href = link.hef
    return (rd_href, rd_lookup_href, rd_group_href)

@asyncio.coroutine
def main():
    protocol = yield from Context.create_client_context()

    request = Message(code=GET)
    request.set_request_uri('coap://[fd28::1]/.well-known/core')

    # get the url path for the RD lookup resource 
    try:
        response = yield from protocol.request(request).response
    except Exception as e:
        print('Failed to fetch resource:', e)
        sys.exit(1)
    link = response.payload.decode('utf-8')
    print('WKC on dir is ', link)
    (rd_href, rd_lookup_href, rd_group_href) = \
                  get_resource_directory_urls(link_header.parse(link))
    print('RD lookup is ', rd_lookup_href)

 
    # check the temp sensor is okay
    request = Message(code=GET)
    #request.set_request_uri('coap://[fd28::1]/.well-known/core?rt=rd.core')
    request.set_request_uri('coap://[fd28::2]/temperature')

    try:
        response = yield from protocol.request(request).response
    except Exception as e:
        print('Failed to fetch resource:', e)
        sys.exit(1)
    
    print('Result: %s\n%r'%(response.code, cbor.loads(response.payload)))


    # Find the remote resources in the RD
    request = Message(code=GET)
    request.set_request_uri('coap://[fd28::1]' + rd_lookup_href + '/res')

    try:
        response = yield from protocol.request(request).response
    except Exception as e:
        print('Failed to fetch resource:', e)
        sys.exit(1)
    
    print('Result: %s\n%r'%(response.code, response.payload.decode('utf-8')))


    # now find a temperature sensor
    links = link_header.parse(response.payload.decode('utf-8'))
    for link in links.links:
        for (tag, value) in link.attr_pairs:
            if tag == 'rt' and value == 'jan.newmarch:temperature-sensor':
                # we've got a temp sensor, look it up
                href = link.href
                request = Message(code=GET)
                print('Request uri is "%s"' % href)
                request.set_request_uri(href)
                try:
                    response = yield from protocol.request(request).response
                except Exception as e:
                    print('Failed to fetch resource:', e)
                    sys.exit(1)
    
                print('Result: %s\n%r'%(response.code, cbor.loads(response.payload)))

if __name__ == "__main__":
    asyncio.get_event_loop().run_until_complete(main())

