#!/usr/bin/env python3

import asyncio

import aiocoap.resource as resource
import aiocoap
#from aiocoap import *
import re
import cbor

CONTENT_FORMAT_CBOR = 60
CONTENT_FORMAT_CORE = 40

from subprocess import PIPE, Popen

class TemperatureResource(resource.Resource):
    def __init__(self):
        super(TemperatureResource, self).__init__()
        self.rt = 'jan.newmarch.name:temperature-sensor'
        self.if_ = 'https:/jan.newmarch.name/temperature-sensor'


    @asyncio.coroutine
    def render_get(self, request):
        process = Popen(['vcgencmd', 'measure_temp'], stdout=PIPE)
        output, _error = process.communicate()
        list = re.split("[='\n]", output.decode('utf-8'))
        dict = {'temperature' : list[1], 'unit' : list[2]}
        mesg = aiocoap.Message(code=aiocoap.CONTENT,
                               payload=cbor.dumps(dict))
        mesg.opt.content_format = CONTENT_FORMAT_CBOR
        return mesg
    
def register_resource():
    print('Registering resource')
    context = yield from aiocoap.Context.create_client_context()
    print('Got context')

    payload = '</temperature>;' +\
              'rt="jan.newmarch:temperature-sensor";' +\
              'if="https://jan.newmarch.name/temperature-sensor"'
    request = aiocoap.Message(code=aiocoap.POST,
                              payload=payload.encode('utf-8',))
    request.set_request_uri('coap://[fd28::1]/rd?ep="RPi in my study"&con="coap://[fd28::2]"')

    print('Prepared msg')
    # create the new resource on the directory
    response = yield from context.request(request).response
    created_uri =  str(response.payload.decode('utf-8'))
    print('Result: %s\n%r'%(response.code, created_uri))
 
#@asyncio.coroutine    
def main():
    # Resource tree creation
    root = resource.Site()
    
    root.add_resource(('temperature',), TemperatureResource())
    root.add_resource(('.well-known', 'core'),
                          resource.WKCResource(root.get_resources_as_linkheader))
    # register our resources with the directory
    asyncio.get_event_loop().run_until_complete(register_resource())
    
    # now run the server forever
    asyncio.async(aiocoap.Context.create_server_context(root))
    asyncio.get_event_loop().run_forever()

if __name__ == "__main__":
    main()
