#!/usr/bin/env python3

import asyncio
import cbor
from aiocoap import *

CONTENT_FORMAT_CBOR = 60

async def main():
    protocol = await Context.create_client_context()

    request = Message(code=GET, uri='coap://[fd28::2]/temperature', observe=0)

    pr = protocol.request(request)

    # Note that it is necessary to start sending
    r = await pr.response
    print("Initial temperature: %r"%(cbor.loads(r.payload)))

    async for r in pr.observation:
        if (r.opt.content_format == CONTENT_FORMAT_CBOR): 
            print("New temperature: %r"%(cbor.loads(r.payload)))
        else:
            print('Unknown format')

if __name__ == "__main__":
    asyncio.get_event_loop().run_until_complete(main())
