
import asyncio

import aiocoap.resource as resource
from aiocoap.resource import ObservableResource
import aiocoap
import re
import cbor

CONTENT_FORMAT_CBOR = 60
CONTENT_FORMAT_CORE = 40

from subprocess import PIPE, Popen

class TemperatureResource(resource.ObservableResource):
    def __init__(self):
        super(TemperatureResource, self).__init__()
        self.temp = {'temperature': None, 'units': None}
        self.notify() # start monitoring resource state

    def current_temperature(self):
        process = Popen(['vcgencmd', 'measure_temp'], stdout=PIPE)
        output, _error = process.communicate()
        list = re.split("[='\n]", output.decode('utf-8'))
        dict = {'temperature' : float(list[1]), 'unit' : list[2]}
        return dict

    def temperature_changed(self, new_temp):
        if (new_temp['temperature'] != self.temp['temperature']):
            return True
        if (new_temp['unit'] != self.temp['unit']):
            return True
        return False
    
    def notify(self):
        new_temp = self.current_temperature()
        if (self.temperature_changed(new_temp)):
            self.temp = new_temp
            self.updated_state() # inform subscribers
        else:
            print('No temp change')

        # check again after 5 seconds
        asyncio.get_event_loop().call_later(5, self.notify)

    @asyncio.coroutine
    def render_get(self, request):
        print('Render get requested')
        mesg = aiocoap.Message(code=aiocoap.CONTENT,
                               payload=cbor.dumps(self.temp))
        mesg.opt.content_format = CONTENT_FORMAT_CBOR
        return mesg
    
def main():
    # Resource tree creation
    root = resource.Site()
    
    root.add_resource(('temperature',), TemperatureResource())
     
    asyncio.async(aiocoap.Context.create_server_context(root))
    asyncio.get_event_loop().run_forever()

if __name__ == "__main__":
    main()

