#!/usr/bin/env python3

import asyncio

import aiocoap.resource as resource
import aiocoap
import re
import cbor

CONTENT_FORMAT_CBOR = 60
CONTENT_FORMAT_CORE = 40

from subprocess import PIPE, Popen

class TemperatureResource(resource.Resource):
    def __init__(self):
        super(TemperatureResource, self).__init__()
        self.rt = 'jan.newmarch.name:temperature-sensor'
        self.if_ = 'https:/jan.newmarch.name/temperature-sensor'


    @asyncio.coroutine
    def render_get(self, request):
        process = Popen(['vcgencmd', 'measure_temp'], stdout=PIPE)
        output, _error = process.communicate()
        list = re.split("[='\n]", output.decode('utf-8'))
        dict = {'temperature' : float(list[1]), 'unit' : list[2]}
        mesg = aiocoap.Message(code=aiocoap.CONTENT,
                               payload=cbor.dumps(dict))
        mesg.opt.content_format = CONTENT_FORMAT_CBOR
        return mesg

def main():
    # Resource tree creation
    root = resource.Site()
    
    root.add_resource(('.well-known', 'core'), WKCResource(root))
    root.add_resource(('temperature',), TemperatureResource())
     
    asyncio.async(aiocoap.Context.create_server_context(root))

    asyncio.get_event_loop().run_forever()

if __name__ == "__main__":
    main()

